//+------------------------------------------------------------------+
//Market Sessions Indicator GPL
//+------------------------------------------------------------------+

#property indicator_chart_window

//--------------------------------------
extern int    NumberOfDays = 500;        // 
extern string AsiaBegin    = "03:00";   // 
extern string AsiaEnd      = "12:00";   // 
extern color  AsiaColor=MidnightBlue; // 
extern bool   AsiaExtend   = false; // 
extern string EurBegin     = "10:00";   // 
extern string EurEnd       = "19:00";   // 
extern color  EurColor= DarkKhaki;       // 
extern bool   EurExtend   = false; // 
extern string USABegin     = "15:00";   // 
extern string USAEnd       = "00:00";   // 
extern color  USAColor=DarkBlue; // 
extern bool   USAExtend   = true; // 
extern bool   GapCorrection   = true; // 
extern long   GapTimeSpan   = 262800; // Corresponds to 3 days and 1 hour
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void init() 
  {
   DeleteObjects();
   for(int i=0; i<NumberOfDays; i++) 
     {
      CreateObjects("AS"+i,AsiaColor);
      CreateObjects("EU"+i,EurColor);
      CreateObjects("US"+i,USAColor);
     }
   Comment("");
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
void deinit() 
  {
   DeleteObjects();
   Comment("");
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CreateObjects(string no,color cl) 
  {
   ObjectCreate(no,OBJ_RECTANGLE,0,0,0,0,0);
   ObjectSet(no,OBJPROP_STYLE,STYLE_DASH);
   ObjectSet(no,OBJPROP_COLOR,cl);
   ObjectSet(no,OBJPROP_BACK,True);
   ObjectSet(no,OBJPROP_WIDTH,1);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void DeleteObjects() 
  {
   for(int i=0; i<NumberOfDays; i++) 
     {
      ObjectDelete("AS"+i);
      ObjectDelete("EU"+i);
      ObjectDelete("US"+i);
     }
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
void start() 
  {
   datetime dt=CurTime();
   int weekday=TimeDayOfWeek(dt);
   for(int i=0; i<NumberOfDays; i++) 
     {
      while(weekday==0) { dt=decDateTradeDay(dt); 
         weekday=TimeDayOfWeek(dt);
         }
      if (weekday==1) {
      if (!AsiaExtend) DrawObjects(dt,"AS"+i,AsiaBegin,AsiaEnd);
      if (!EurExtend) DrawObjects(dt,"EU"+i,EurBegin,EurEnd);
      if (!USAExtend) DrawObjects(dt,"US"+i,USABegin,USAEnd);
      }
      if (weekday>1 && weekday<6) {
      DrawObjects(dt,"AS"+i,AsiaBegin,AsiaEnd);
      DrawObjects(dt,"EU"+i,EurBegin,EurEnd);
      DrawObjects(dt,"US"+i,USABegin,USAEnd);
      }
      if (weekday==6) {
      if (AsiaExtend) DrawObjects(dt,"AS"+i,AsiaBegin,AsiaEnd);
      if (EurExtend) DrawObjects(dt,"EU"+i,EurBegin,EurEnd);
      if (USAExtend) DrawObjects(dt,"US"+i,USABegin,USAEnd);
      }
      dt=decDateTradeDay(dt);
      weekday=TimeDayOfWeek(dt);
         
            
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void DrawObjects(datetime dt,string no,string tb,string te) 
  {
   datetime t1,t2,t3,t4;
   double   p1,p2;
   int      b1,b2;
   long     d1;
      
   if(StringFind(no,"US",0)==0 && USAExtend ) {t1=ExtendDate(dt,tb);}
   else if (StringFind(no,"EU",0)==0 && EurExtend ) {t1=ExtendDate(dt,tb);}
   else if (StringFind(no,"AS",0)==0 && AsiaExtend ) {t1=ExtendDate(dt,tb);}
   else {
      t1=StrToTime(TimeToStr(dt,TIME_DATE)+" "+tb);
     }
   t2=StrToTime(TimeToStr(dt, TIME_DATE)+" "+te);
   b1=iBarShift(NULL, 0, t1);
   b2=iBarShift(NULL, 0, t2);
   t3=iTime(NULL, 0, b1);
   t4=iTime(NULL, 0, b2);
   
   if (GapCorrection && GapTimeSpan>0) { 
      d1=t4-t3;
      if ( d1>0 && d1>=GapTimeSpan) { return; }
   }
   p1=High[Highest(NULL, 0, MODE_HIGH, b1-b2+1, b2)];
   p2=Low [Lowest (NULL, 0, MODE_LOW , b1-b2+1, b2)];
   ObjectSet(no,OBJPROP_TIME1,t3);
   ObjectSet(no,OBJPROP_PRICE1,p1);
   ObjectSet(no,OBJPROP_TIME2,t4);
   ObjectSet(no,OBJPROP_PRICE2,p2);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
datetime decDateTradeDay(datetime dt) 
  {
   int ty=TimeYear(dt);
   int tm=TimeMonth(dt);
   int td=TimeDay(dt);
   int th=TimeHour(dt);
   int ti=TimeMinute(dt);

   td--;
   if(td==0) 
     {
      tm--;
      if(tm==0) 
        {
         ty--;
         tm=12;
        }
      if(tm==1 || tm==3 || tm==5 || tm==7 || tm==8 || tm==10 || tm==12) td=31;
      if(tm==2) if(MathMod(ty, 4)==0) td=29; else td=28;
      if(tm==4 || tm==6 || tm==9 || tm==11) td=30;
     }
   return(StrToTime(ty+"."+tm+"."+td+" "+th+":"+ti));
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
datetime ExtendDate(datetime dt,string tb) 
  {
   int ty=TimeYear(dt);
   int tm=TimeMonth(dt);
   int td=TimeDay(dt);
   int th=TimeHour(dt);
   int ti=TimeMinute(dt);

   td--;
   if(td==0) 
     {
      tm--;
      if(tm==0) 
        {
         ty--;
         tm=12;
        }
      if(tm==1 || tm==3 || tm==5 || tm==7 || tm==8 || tm==10 || tm==12) td=31;
      if(tm==2) if(MathMod(ty, 4)==0) td=29; else td=28;
      if(tm==4 || tm==6 || tm==9 || tm==11) td=30;
     }
   return(StrToTime(ty+"."+tm+"."+td+" "+tb));
  }
//+------------------------------------------------------------------+
